// *******************************************************************************
// File name:		algorthm.h
// Author:		Laura Dekker
// Description:
//
//
// Change history:
//
//
// *******************************************************************************

#ifndef _ALGORTHM_H
#define _ALGORTHM_H

#include <fstream.h>
#include "qgtypes.h"


enum StatsLevel { NONE, BEST_ONLY, POST_SELECTION_POOLS, ALL_STAGES_POOLS };


class Algorithm
{
public:
	Algorithm(void);
	virtual ~Algorithm(void);

        // Algorithm run
	virtual void Run(void);
       
	virtual void StartGA(void);
	virtual void Iteration(void) = 0;
	virtual int EndConditions(void) = 0;
	virtual void EndGA(void);
	virtual void ResetGA(void) = 0;

	long GetIteration(void);
	void SetIteration(long);
	long GetMaxIterations(void);
	void SetMaxIterations(long);

	// Pool management
	void SetPool(int index, Pool* pool);
	Pool* GetPool(int index);
	void SetNumPools(int num_pools);
	int GetNumPools(void);

        // Evaluator management
	void SetEvaluator(Evaluator* evaluator);
	Evaluator* GetEvaluator(void);
	static Evaluator* _evaluator;
	void SetEvaluatorType(EvaluatorType type);
	EvaluatorType GetEvaluatorType(void);

	void SetOptimiseType(OptimiseType min_or_max);
	OptimiseType GetOptimiseType(void) const;

	Individual* GetBestIndividual(void);
	
	void SetOutputFile(char* path);
	char* GetOutputFile(void);
	void SetOutputDetail(StatsLevel detail);
	StatsLevel GetOutputDetail(void);
	void SetOutputFrequency(int frequency);
	int GetOutputFrequency(void);

protected:
	Pool* _pool_array[MAX_POOLS];
	int _num_pools;

	ofstream output;
	char _output_file[MAX_FILE_PATH];
	StatsLevel _output_detail;
	int _output_frequency;

	long _iteration;
	long _max_iterations;

	EvaluatorType _evaluator_type;
	OptimiseType _optimise;
};

#endif

