/****************************************************************
 *								*
 * Program to 
 *								*
 * File: schedall.c	main code      				*
 *								*
 * Author: W. Langdon 
 *								*
 * Started: 5 December 1994 					*
 *								*
 ****************************************************************/

#define version "5 Dec 1994 $Revision: 1.1 $"

//Includes main function*/

#include <stdio.h>
#include <string.h>
#include <assert.h>
#include <float.h>
#include <time.h>
#include "gridalg.h"
#include "grideval.h"

extern ofstream debug;

/*Modifications (in reverse order)

WBL 23 Oct 94  
*/


/* This function reads the network parameters in from the study
 * file. It is however very simple, and may NOT detect errors 
 * within the study file */
void ReadParameters (char filename[32], 
int* nn, int* nl, int*nw, float P[NW][NN], int* NNLK, int* NNLJ, float* XL,
float* LINERATING )
{
	FILE *record;
	int w, i;

/* filename is provided by GetStudyName below */	
	record = fopen(filename,"r");

	fscanf(record,"%i %i %i", nn, nl, nw);

	assert(*nn<NN);
	assert(*nl<NL);
	assert(*nw<NW);
	assert(*nl<*nw);
	assert(*nw<=9);

	for (w=1; w<=*nw; w++) {
	for (i=1; i<=*nn; i++) {
	   fscanf(record,"%f", &P[w][i]);
	}
	}

	for (i=1; i<=*nl; i++) {
	   char c[80]; 
	   fscanf(record,"%i %i %f %f %*s",
		  &NNLK[i], &NNLJ[i], &XL[i], &LINERATING[i], c);
/*	   MAINT[i] = (c[0]=='m'||c[0]=='M') ? TRUE : FALSE; */
	}
	
	fclose(record);

	return;
}

/* Function obtains a study file name from the program operator */
void GetStudyName (char filename[32], int* start, int* end )
{
	FILE *record;

	printf ("\nEnter name of study file: ");
	scanf ("%s",filename);
	printf ("\n");

/* Checks to ensuree the file name given is valid */
	record = fopen(filename,"r");

	if (record == NULL) {
	   printf ("\nerror: failed to open study file '%s'\n",filename);
	   exit (0);
	}
	printf ("Data taken from study file '%s'\n",filename);
/*	printf ("\nEnter first and last weeks for line 1 (-ve => permutes): ");
**	scanf ("%i %i", start, end);
*/
	printf ("\n");
}

void display_maint(int nl, int nw, BOOL MAINT[NW][NL], 
		   float fit, int maint, float total_c, float total_shed) {
int j, k;
int total_maint = 0;
char buf[80];

	for(j=1;j<=nw;j++)
	 {
		 for(k=1;k<=nl;k++) 
			 if(MAINT[j][k]) {cout<<k; total_maint++;}
		 if(j<nw) cout<<",";
         }
	sprintf(buf,"\t%6.0f\t%d\t%6.0f\t%6.0f\n", 
	       fit, maint, total_c, total_shed );
        cout<<buf;
}

/**********************************************************************/
/* Routines and data shared by main                                   */
/**********************************************************************/

/* The globals below store the network parameters */
int NNLK[NL], NNLJ[NL], nn, nl, nw;
int order[NL];
float LINERATING[NL];

float P[NW][NN];

/* The globals below are used for storing the output data */
float XL[NL], FLOW[NL];
float IMBALANCE[NN];

#define NC 2000

int nfit[NC], ncosts[NC], nshed[NC];
int nfit_high = 0;
int ncosts_high = 0;
int nshed_high  = 0;
int nfit_overflow = 0;
int ncosts_overflow = 0;
int nshed_overflow  = 0;
float fit_min = FLT_MAX;
float fit_max = -FLT_MAX;
float total_c_min = FLT_MAX;
float total_c_max = -FLT_MAX;
const float fit_overflow = 310000; /* > 3800*8*9 + 8*4000 */
const float total_c_overflow = 300000; /* > 3800*8*9 */
/*float total_s_min = 345.1; /* FLT_MAX */
float total_s_max = -FLT_MAX;
const float total_s_overflow = 40000; /* > 3800 * 9 */
const float permute_worst = 3870; /*from earlier runs*/

void display_stats() {
int i,sum;

sum = 0;
printf("\nFitness\tlowest %f\tHighest %f\n", fit_min, fit_max);
printf("\nMW weeks  number of times\n");
for(i=0;i<NC;i++) if(nfit[i] != 0) {
	printf("%6.0f\t%4d\n",i*5.0,nfit[i]); sum+=nfit[i];}
if(nfit_high>0) {
	printf("%6.0f<\t%4d or < -0.01\n",(NC-1)*5.0, nfit_high); 
        sum+=nfit_high;}
if(nfit_overflow>0) {
	printf("%6.0f<\t%4d or < -0.01\n",fit_overflow, nfit_overflow); 
        sum+=nfit_overflow;}
printf("\t----\n%12d\n",sum);

sum = 0;
printf("\nLine costs\tlowest %f\tHighest %f\n", total_c_min, total_c_max);
printf("\nMW weeks  number of times\n");
for(i=0;i<NC;i++) if(ncosts[i] != 0) {
	printf("%6.0f\t%4d\n",i*5.0,ncosts[i]); sum+=ncosts[i];}
if(ncosts_high>0) {
	printf("%6.0f<\t%4d or < -0.01\n",(NC-1)*5.0, ncosts_high); 
        sum+=ncosts_high;}
if(ncosts_overflow>0) {
	printf("%6.0f<\t%4d or < -0.01\n",total_c_overflow, ncosts_overflow); 
        sum+=ncosts_overflow;}
printf("\t----\n%12d\n",sum);

sum = 0;
printf("\nLoad shedding\t               \tHighest %f\n", total_s_max);
printf("    MW\tnumber of times\n");
for(i=0;i<NC;i++) if(nshed[i] != 0 ) {
	printf("%6.0f\t%4d\n",i*5.0,nshed[i]); sum+=nshed[i];}
if(nshed_high>0) {
	printf("%6.0f<\t%4d or < -0.01\n",(NC-1)*5.0, nshed_high); 
        sum+=nshed_high;}
if(nshed_overflow>0) {
	printf("%6.0f<\t%4d or < -0.01\n",total_s_overflow, nshed_overflow); 
        sum+=nshed_overflow;}
printf("\t----\n%12d\n",sum);
}

/*
void dummycost() {
	int j, k;
	for(j=1;j<=nw;j++)
	 {
		 for(k=1;k<=nl;k++) 
			 {if(MAINT[j][k])printf("%1d",k);}
		 printf(",");
         }
	printf(".\n");
}/*end dummycost*/

void recurse ( int line, const int* available ) {
}/*end recurse*/

float fit ( int order[NL] ) {

BOOL MAINT[NW][NL];

int nmaint;
float f;
float total_c;
float total_shed;

cout<<"[";
for (int i=1;i<=nl;i++) cout<<order[i];
cout<<"] = "<< flush;

        Schedule (nn, nl, nw, P, NNLK, NNLJ, XL, LINERATING, order, 
		  MAINT);

        f = Fitness(nn, nl, nw, P, NNLK, NNLJ, XL, LINERATING, MAINT, 
		      &nmaint, &total_c, &total_shed );

        display_maint(nl, nw, MAINT, f, nmaint, total_c, total_shed);

	return f;
}

//#ifdef DEBUG1
ofstream debug;
//#endif DEBUG1



/* Performs program as described by above functions */		
int main() {
time_t start, now, end;
char filename[32];

int dummy;

cout<<"Program to run `greedy' GA "
    << "Version "<< version<<endl;

memset(ncosts,0,sizeof(ncosts));
memset(nshed,0,sizeof(nshed));

GetStudyName (filename, &dummy, &dummy);

ReadParameters (filename, &nn, &nl, &nw, P, NNLK, NNLJ, XL, LINERATING);

time (&start);
cout<<"Started at "<< ctime(&start) << endl;

int generation = 0;

GridGA ourga;

cout<<"StartUP"<<endl;
ourga.StartUp();

while (!ourga.EndConditions()) {
	time (&now);
	cout<<"Starting gen "<<++generation <<"  at "<<ctime(&now)<<endl;
	ourga.Iteration();

cout<<"Best individual ";
cout<<ourga.GetBestIndividual()<<" "<<flush;
//ourga.GetBestIndividual()->PrintGenotype(cout);
cout<<endl;

}

cout<<"Best individual ";
cout<<ourga.GetBestIndividual()<<" "<<flush;
//ourga.GetBestIndividual()->PrintGenotype(cout);
cout<<endl;

ourga.EndGA();

time (&end);
cout<<"took "<<end - start <<" secs. Finished at "<<ctime(&end)<<endl;

return 1;
}
