// *******************************************************************************
// File name:		individ.h
// Author:		Laura Dekker
// Description:
//
//
// Change history:
//
//
// *******************************************************************************

#ifndef _INDIVID_H
#define _INDIVID_H


#include <iostream.h>
#include "qgtypes.h"




class Gene
{
friend class Pool;
friend ostream& operator<< (ostream&, const Gene&);
friend Gene& operator>> (istream&, Gene&);
public:
	Gene(void);
	Gene(const Gene&);
	Gene& operator=(const Gene&);
	Gene& operator=(const double);
	virtual ~Gene(void);
	operator int();
	operator long();
	operator double();
	
	void SetMin(double);
	double GetMin(void);
	void SetMax(double);
	double GetMax(void);

protected:
	double _value;
	double _min;
	double _max;
};




struct Location
{
	long _x;
	long _y;
};




class Chromosome
{
friend class Individual;
friend class Pool;
friend ostream& operator<< (ostream&, const Chromosome&);
friend Chromosome& operator>> (istream&, Chromosome&);

public:
	Chromosome(void);
	Chromosome(long num_genes, GeneType type);
	Chromosome(long num_genes, char alphabet);
	Chromosome(const Chromosome&);
	Chromosome& operator=(const Chromosome&);
	virtual ~Chromosome(void);

	unsigned long BinaryDecode(void) const;
	void BinaryEncode(unsigned long value);
	unsigned long BaseDecode(void) const;
	void BaseEncode(unsigned long value);
	void ConvertBase(char new_alphabet);

	double ReadGene(long index);
	void WriteGene(long index, double value);

	void SetMinValue(double);
	double GetMinValue(void);
	void SetMaxValue(double);
	double GetMaxValue(void);

	char GetAlphabet(void);

protected:
	GeneType _type;
	char _alphabet;
	double _min;
	double _max;
	long _num_genes;
	void* _gene_array;
};




#ifdef EXTERNAL_FITNESS
class _export Individual
#else
class Individual
#endif
{
friend class Pool;
friend ostream& operator<< (ostream&, const Individual&);
friend Individual& operator>> (istream&, Individual&);

public:
	Individual(void);
	Individual(long num_chromosomes);
	Individual(long num_chromosomes, long num_genes, GeneType type);
	Individual(long num_chromosomes, long num_genes, char alphabet);
	Individual(const Individual&);
	Individual& operator=(const Individual&);
	virtual ~Individual(void);

	long GetPoolIndex(void);
	long GetNumChromosomes(void);
	void SetNumChromosomes(long);
	long GetNumGenes(long chromosome);
	GeneType GetGeneType(long chromosome);
	double GetMinValue(long chromosome);
	double GetMaxValue(long chromosome);
	void SetMinValue(long chromosome, double value);
	void SetMaxValue(long chromosome, double value);

	char GetAlphabet(long chromosome);

	void AddChromosome(long index, Chromosome* chromosome);

	double ReadChromosome(long chromosome);
	
	double ReadGene(long chromosome, long gene);

	ostream& PrintGenotype(ostream&);
	ostream& PrintPhenotype(ostream&);

	FitnessType GetFitness(void);

	Location GetLocation(void);
	void SetLocation(Location);

protected:
	void WriteChromosome(long chromosome, double value);

	long _num_chromosomes;
	Chromosome** _chromosome_array;

	int _fitness_valid;
	FitnessType _fitness_cache;

	long _pool_index;
	Location _location;
};


#ifdef EXTERNAL_FITNESS
extern "C" FitnessType FAR PASCAL EvaluateFitness(Individual FAR&);
#else
extern FitnessType EvaluateFitness(Individual&);
#endif


#endif
