// *******************************************************************************
// File name:		pools.h
// Author:		Laura Dekker
// Description:
//
//
// Change history:
//
//
// *******************************************************************************

#ifndef _POOLS_H
#define _POOLS_H


#include <iostream.h>
#include "qgtypes.h"

 

#define	MAX_GENES			0

class Pool
{
friend ostream& operator<< (ostream&, const Pool&);
friend Pool& operator>> (istream&, Pool&);

public:
			Pool(void);
			Pool(long num_individuals);
			Pool(long num_individuals, long num_chromosomes, long num_genes, GeneType type);
			Pool(long num_individuals, long num_chromosomes, long num_genes, char alphabet);
			Pool(long num_individuals, Individual* prototype);
			Pool(const Pool&);
			Pool& operator=(const Pool&);
			virtual ~Pool(void); 

	long            GetNumIndividuals(void);
	Individual*     GetIndividualHandle(long index);
	Individual*     GetIndividualCopy(long index);
	void            AddIndividual(long index, Individual* individual);
	void            DeleteIndividual(long index);
	Individual*		RemoveIndividual(long index);

	long            GetNumChromosomes(long individual);
	Chromosome*     GetChromosomeHandle(long individual, long chromosome);
	Chromosome*     GetChromosomeCopy(long individual, long chromosome);
	double          ReadChromosome(long individual, long chromosome);
	void            WriteChromosome(long individual, long chromosome, double value);
	void            AddChromosome(long individual, long chromosome, Chromosome* new_chromosome);
	void            DeleteChromosome(long individual, long chromosome);

	long            GetNumGenes(long individual, long chromosome);
	double          ReadGene(long individual, long chromosome, long gene);
	void            WriteGene(long individual, long chromosome, long gene, double value);

	void		SetMinValue(long individual, long chromosome, double min);
	void		SetMinValue(long individual, long chromosome, long gene, double min);
	double		GetMinValue(long individual, long chromosome);
	double		GetMinValue(long individual, long chromosome, long gene);
	void		SetMaxValue(long individual, long chromosome, double max);
	void		SetMaxValue(long individual, long chromosome, long gene, double max);
	double		GetMaxValue(long individual, long chromosome);
	double		GetMaxValue(long individual, long chromosome, long gene);

	char		GetAlphabet(long individual, long chromosome);

	GeneType	GetGeneType(long individual, long chromosome);

	void		SortPool(void);
	void		JumblePool(void);

	FitnessType	GetFitness(long index);
	void		InvalidateFitness(long index);
	FitnessType	GetTotalFitness(void);
	FitnessType	GetAverageFitness(void);
	long 		GetHighestFitness(OptimiseType min_or_max);
	long 		GetLowestFitness(OptimiseType min_or_max);
	long		GetNthHighest(long nth, OptimiseType min_or_max);

	void		SetOldAvgFitness(FitnessType);
	FitnessType	GetOldAvgFitness(void);
	void		StoreOverallBest(OptimiseType min_or_max);
	Individual*	GetOverallBest(void);

	ostream&	Statistics(ostream& os);
	ostream& 	Fitnesses(ostream& os);

protected:
	Individual**    _individual_array;

	long            _num_individuals;

	long*           _order_array;
	int             _order_valid;

	FitnessType		_old_avg_fitness;
	Individual*		_overall_best;
};



void    CopyIndividual(Pool* src, long si, Pool* dest, long di);

void    SwapGenes  (Pool* src, long si, long sc,
					Pool* dest, long di, long dc,
					long swap_index, long num_genes = MAX_GENES);




#endif
